@@ -24,9 +24,8 @@ module Text.CSL.Eval
 import Control.Arrow
 import Control.Applicative ( (<$>) )
 import Control.Monad.State
-import Data.Char
+import Data.Char ( toLower, isDigit, isLetter )
 import Data.List
-import qualified Data.Map as M
 import Data.Maybe
 
 import Text.CSL.Eval.Common
@@ -36,7 +35,6 @@ import Text.CSL.Eval.Names
 import Text.CSL.Output.Plain
 import Text.CSL.Reference
 import Text.CSL.Style
-import Text.Pandoc.Definition
 
 -- | Produce the output with a 'Layout', the 'EvalMode', a 'Bool'
 -- 'True' if the evaluation happens for disambiguation purposes, the
@@ -52,7 +50,7 @@ evalLayout (Layout _ _ es) em b l m o a r
                         then []
                         else [noOutputError]
                   x | title r == citeId cit ++ " not found!" -> [noBibDataError $ cit]
-                    | otherwise                              -> x
+                    | otherwise                              -> suppTC x
       locale = case l of
                  [x] -> x
                  _   -> Locale [] [] [] [] []
@@ -60,9 +58,14 @@ evalLayout (Layout _ _ es) em b l m o a r
       cit    = case em of
                  EvalCite    c -> c
                  EvalSorting c -> c
-                 EvalBiblio  s -> emptyCite { citePosition = s }
-      initSt = EvalState (mkRefMap r) (Env cit (localeTermMap locale) m
+                 EvalBiblio  c -> c
+      initSt = EvalState (mkRefMap r) (Env cit (localeTerms locale) m
                          (localeDate locale) o [] a) [] em b False [] [] False [] [] []
+      suppTC = let getLang = take 2 . map toLower in
+               case (getLang $ localeLang locale, getLang $ language r) of
+                 (_,  "en") -> id
+                 ("en", []) -> id
+                 _          -> proc' rmTitleCase
 
 evalSorting :: EvalMode -> [Locale] -> [MacroMap] -> [Option] ->
                [Sort] -> [Abbrev] -> Reference -> [Sorting]
@@ -98,7 +101,7 @@ evalElement el
     | Choose i ei e         <- el = evalIfThen i ei e
     | Macro    s   fm       <- el = return . appendOutput fm =<< evalElements =<< getMacro s
     | Const    s   fm       <- el = return $ rtfParser fm s
-    | Number   s f fm       <- el = formatNumber f fm =<< getStringVar s
+    | Number   s f fm       <- el = formatNumber f fm s =<< getStringVar s
     | Variable s f fm d     <- el = return . addDelim d =<< concatMapM (getVariable f fm) s
     | Group        fm d l   <- el = when' ((/=) [] <$> tryGroup l) $
                                     return . outputList fm d =<< evalElements l
@@ -120,7 +123,16 @@ evalElement el
                                     evalElement $ Names rs ns nfm (d' `betterThen` d) []
                              _   -> evalElement e
 
-      tryGroup     l = get >>= \s -> evalElements (rmTermConst l) >>= \r -> put s >> return r
+      tryGroup l = if hasVar l
+                   then get >>= \s -> evalElements (rmTermConst l) >>= \r -> put s >> return r
+                   else return [ONull]
+      hasVar  = not . null . query hasVarQ
+      hasVarQ e
+          | Variable {} <- e = [e]
+          | Date     {} <- e = [e]
+          | Names    {} <- e = [e]
+          | Number   {} <- e = [e]
+          | otherwise        = []
       rmTermConst [] = []
       rmTermConst (e:es)
           | Term  {} <- e = rmTermConst es
@@ -129,24 +141,24 @@ evalElement el
 
       ifEmpty p t e = p >>= \r -> if r == [] then t else return (e r)
 
-      withNames e n f = modify (\s -> s { authSub = concat e
+      withNames e n f = modify (\s -> s { authSub = e ++ authSub s
                                         , env = (env s)
                                           {names = n : names (env s)}}) >> f >>= \r ->
-                        modify (\s -> s { authSub = []
+                         modify (\s -> s { authSub = filter (not . flip elem e) (authSub s)
                                         , env = (env s)
                                           {names = tail $ names (env s)}}) >> return r
 
       getFirst        [] = return []
-      getFirst    (x:xs) = whenElse ((/=) []  <$> evalElement x)
-                                    (consuming $  evalElement x)
+      getFirst    (x:xs) = whenElse ((/=) []  <$> substituteWith x)
+                                    (consuming $  substituteWith x)
                                     (getFirst xs)
-
       getMacro         s = maybe [] id . lookup s <$> gets (macros . env)
-      getVariable f fm s = case (map toLower s) of
+      getVariable f fm s = if isTitleVar s || isTitleShortVar s
+                           then consumeVariable s >> formatTitle s f fm else
+                           case (map toLower s) of
                              "year-suffix" -> getStringVar "ref-id" >>= \k  ->
                                               return . return $ OYearSuf [] k [] fm
                              "page"        -> getStringVar "page" >>= formatRange fm
-                             "title"       -> formatTitle f fm
                              "locator"     -> getLocVar >>= formatRange fm . snd
                              "url"         -> getStringVar "url" >>= \k ->
                                               if null k then return [] else return [OUrl (k,k) fm]
@@ -188,7 +200,9 @@ evalIfThen i ei e
       chkType         t = let chk = (==) (formatVariable t) . show . fromMaybe NoType . fromValue
                           in  getVar False chk "ref-type"
       chkNumeric      v = do val <- getStringVar v
-                             return (v `elem` numericVars && or (map isDigit val))
+                             as  <- gets (abbrevs . env)
+                             let val' = if getAbbreviation as v val == [] then val else getAbbreviation as v val
+                             return (isNumericString val')
       chkDate         v = getDateVar v >>= return . not . null . filter ((/=) [] . circa)
       chkPosition     s = if s == "near-note"
                           then gets (nearNote . cite . env)
@@ -214,42 +228,46 @@ getFormattedValue o as f fm s val
     | otherwise                                  = []
     where
       value     = if stripPeriods fm then filter (/= '.') else id
-      getAbbr v = if f /= Short then v else
-                  case lookup "default" as of
-                    Nothing -> v
-                    Just x  -> case lookup s x of
-                                 Nothing -> v
-                                 Just x' -> case M.lookup v x' of
-                                              Nothing  -> v
-                                              Just x'' -> x''
+      getAbbr v = if f == Short
+                  then let ab = getAbbreviation as s v in
+                       if null ab then v else ab
+                  else v
       nameOpts = ("name-as-sort-order","all") : o
       sortDate = [ DatePart "year"  "numeric-leading-zeros" "" emptyFormatting
                  , DatePart "month" "numeric-leading-zeros" "" emptyFormatting
                  , DatePart "day"   "numeric-leading-zeros" "" emptyFormatting]
 
-formatTitle :: Form -> Formatting -> State EvalState [Output]
-formatTitle f fm
-    | Short <- f = getIt "title-short" "title"
-    | otherwise  = getIt "title" "title-short"
+formatTitle :: String -> Form -> Formatting -> State EvalState [Output]
+formatTitle s f fm
+    | Short <- f
+    , isTitleVar      s = try (getIt $ s ++ "-short") $ getIt s
+    | isTitleShortVar s = try (getIt s) $ return . rtfParser fm =<< getTitleShort s
+    | otherwise         = getIt s
     where
-      getIt x fb = do
+      try g h = g >>= \r -> if r == [] then h else return r
+      getIt x = do
         o <- gets (options . env)
         a <- gets (abbrevs . env)
-        r <- getVar [] (getFormattedValue o a f fm x) x
-        case r of
-          [] -> getVar [] (getFormattedValue o a f fm x) fb
-          _  -> return r
+        getVar [] (getFormattedValue o a f fm x) x
 
-formatNumber :: NumericForm -> Formatting -> String -> State EvalState [Output]
-formatNumber f fm s
-    | or (map isDigit s) = do tm <- gets (terms . env)
-                              return . output fm . format tm . filter isDigit $ s
-    | otherwise          = do return . output fm $ s
+formatNumber :: NumericForm -> Formatting -> String -> String -> State EvalState [Output]
+formatNumber f fm v n
+    = gets (abbrevs . env) >>= \as ->
+      if isNumericString (getAbbr as n)
+      then gets (terms . env) >>=
+           return . output fm . flip process (getAbbr as n)
+      else return . output fm . getAbbr as $ n
     where
+      getAbbr       as   = if getAbbreviation as v n == [] then id else getAbbreviation as v
+      checkRange'   ts   = if v == "page" then checkRange ts else id
+      process       ts   = checkRange' ts . printNumStr . map (renderNumber ts) .
+                           breakNumericString . words
+      renderNumber  ts x = if isTransNumber x then format ts x else x
+
       format tm = case f of
-                    Ordinal     -> ordinal tm
-                    LongOrdinal -> longOrdinal tm
-                    Roman       -> if readNum s < 6000 then roman else id
+                    Ordinal     -> ordinal     tm v
+                    LongOrdinal -> longOrdinal tm v
+                    Roman       -> if readNum n < 6000 then roman else id
                     _           -> id
 
       roman     = foldr (++) [] . reverse . map (uncurry (!!)) . zip romanList .
@@ -260,23 +278,66 @@ formatNumber f fm s
 	          ,[ "", "m", "mm", "mmm", "mmmm", "mmmmm"]
                   ]
 
+
+checkRange :: [CslTerm] -> String -> String
+checkRange _ [] = []
+checkRange ts (x:xs) = if x == '-'
+                       then pageRange ts ++ checkRange ts xs
+                       else x             : checkRange ts xs
+
+printNumStr :: [String] -> String
+printNumStr []     = []
+printNumStr (x:[]) = x
+printNumStr (x:"-":y:xs) = x ++ "-"  ++ y ++ printNumStr xs
+printNumStr (x:",":y:xs) = x ++ ", " ++ y ++ printNumStr xs
+printNumStr (x:xs)
+    | x == "-"  = x ++        printNumStr xs
+    | otherwise = x ++ " " ++ printNumStr xs
+
+pageRange :: [CslTerm] -> String
+pageRange = maybe "\x2013" termPlural . findTerm "page-range-delimiter" Long
+
+isNumericString :: String -> Bool
+isNumericString [] = False
+isNumericString s  = null . filter (not . isNumber &&& not . isSpecialChar >>> uncurry (&&)) $
+                     words s
+
+isTransNumber, isSpecialChar,isNumber :: String -> Bool
+isTransNumber = and . map isDigit
+isSpecialChar = and . map (flip elem "&-,")
+isNumber      = filter (not . isLetter) >>> filter (not . flip elem "&-,") >>>
+                map isDigit >>> and &&& not . null >>> uncurry (&&)
+
+breakNumericString :: [String] -> [String]
+breakNumericString [] = []
+breakNumericString (x:xs)
+    | isTransNumber x = x : breakNumericString xs
+    | otherwise       = let (a,b) = break (flip elem "&-,") x
+                            (c,d) = if null b then ("","") else (head' b, tail b)
+                        in filter (/= []) $  a : c : breakNumericString (d : xs)
+
 formatRange :: Formatting -> String -> State EvalState [Output]
 formatRange _ [] = return []
 formatRange fm p = do
   ops <- gets (options . env)
+  ts  <- gets (terms . env)
   let opt = getOptionVal "page-range-format" ops
-      splitRange = break (== '-') >>> trim *** trim . dropWhile (== '-')
-      pages = map splitRange . split (== ',') $ p
-      format a b = if b /= []
-                   then flip Output emptyFormatting [ OStr a emptyFormatting, OPan [Str "\x2013"]
-                                                    , OStr b emptyFormatting]
-                   else flip Output emptyFormatting [ OStr a emptyFormatting]
-      result = case opt of
-                 "expanded" -> map (uncurry format . uncurry expandedRange) pages
-                 "chicago"  -> map (uncurry format . uncurry chicagoRange ) pages
-                 "minimal"  -> map (uncurry format . uncurry minimalRange ) pages
-                 _          -> map (uncurry format                        ) pages
-  return [flip OLoc fm $ addDelim ", " result]
+      pages = tupleRange . breakNumericString . words $ p
+
+      tupleRange [] = []
+      tupleRange (x:"-":[]  ) = return (x,[])
+      tupleRange (x:"-":y:xs) = (x, y) : tupleRange xs
+      tupleRange (x:      xs) = (x,[]) : tupleRange xs
+
+      joinRange (a, []) = a
+      joinRange (a,  b) = a ++ "-" ++ b
+
+      process = case opt of
+                 "expanded" -> checkRange ts . printNumStr . map (joinRange . uncurry expandedRange)
+                 "chicago"  -> checkRange ts . printNumStr . map (joinRange . uncurry chicagoRange )
+                 "minimal"  -> checkRange ts . printNumStr . map (joinRange . uncurry minimalRange )
+                 _          -> checkRange ts . printNumStr . map (joinRange)
+  return [flip OLoc fm $ [OStr (process pages) emptyFormatting]]
 
 expandedRange :: String -> String -> (String, String)
 expandedRange sa [] = (sa,[])
