@@ -47,18 +47,14 @@ evalNames skipEdTrans ns nl d
         p   <- gets (citePosition . cite . env)
         ops <- gets (options . env)
         aus <- gets authSub
-        r   <- if s == "author" && ags /= [] && p == "subsequent" &&
-                  isOptionSet "subsequent-author-substitute" ops
-               then return $ output emptyFormatting $
-                    getOptionVal "subsequent-author-substitute" ops
-               else do res <- agents p            s ags
-                       st  <- get
-                       fb  <- agents "subsequent" s ags
-                       put st
-                       if res /= []
-                          then let role = if aus == "author" then aus ++ "sub" else s
-                               in  return . return . OContrib k role res fb =<< gets etal
-                          else     return []
+        r   <- do res <- agents p            s ags
+                  st  <- get
+                  fb  <- agents "subsequent" s ags
+                  put st
+                  if res /= []
+                    then let role = if aus == ["author"] then concat aus ++ "sub" else s
+                         in  return . return . OContrib k role res fb =<< gets etal
+                    else     return []
         r'  <- evalNames skipEdTrans xs nl d
         num <- gets contNum
         return $ if r /= [] && r' /= []
@@ -332,8 +328,12 @@ formatLabel f fm p s
     | "page"    <- s = checkPlural
     | "volume"  <- s = checkPlural
     | "ibid"    <- s = format' s p
-    | otherwise      = format  s p
+    | otherwise      = if isRole s then form (\fm' x -> [OLabel x fm']) id s p else format s p
     where
+      isRole = flip elem ["author", "collection-editor", "composer", "container-author"
+                         ,"director", "editor", "editorial-director", "editortranslator"
+                         ,"illustrator", "interviewer", "original-author", "recipient"
+                         ,"reviewed-author", "translator"]
       checkPlural = when' (isVarSet s) $ do
                       v <- getStringVar s
                       format  s ('-' `elem` v)
