@@ -41,12 +41,12 @@ xpMods = xpIElem "mods" xpReference
 
 xpReference :: PU Reference
 xpReference
-    = xpWrap ( \ ( ref
-                , (ck,ty,ti,i,d)
-                ,((au,ed,tr),(re,it,pu',dr),(co,ce,dg,om))
-                ,((di',pg,vl,is),(nu,sc,ch))
+    = xpWrap ( \ ((ref,oref)
+                , (ck,(ty,gn),ti,i,d)
+                ,((au,ed,tr,sp),(re,it,pu',dr),(co,ce,dg,om))
+                ,((di',pg,vl,is),(nu,sc,ch,vs))
                 , (di,ac,pu,pp,et)
-                , ((ac',uri),no)
+                , ((ac',uri),ln,st,no)
                  ) ->
                ref { refId            = ck `betterThen` take 10 (concat . words $ fst ti)
                    , refType          = if ty /= NoType then ty else
@@ -63,11 +63,13 @@ xpReference
                    , director         = dr `betterThen` director         ref
                    , collectionEditor = ce `betterThen` collectionEditor ref
                    , publisherPlace   = pp `betterThen` publisherPlace   ref
+                   , numberOfVolumes  = vs `betterThen` numberOfVolumes  ref
                    , containerAuthor  = containerAuthor  ref
                    , url              = uri
                    , note             = no
                    , isbn             = i
                    , doi              = d
+                   , genre            = genre         ref `betterThen` gn
                    , issued           = issued        ref `betterThen` di `betterThen` di'
                    , accessed         = accessed      ref `betterThen` ac `betterThen` ac'
                    , page             = page          ref `betterThen` pg
@@ -77,25 +79,34 @@ xpReference
                    , number           = number        ref `betterThen` nu
                    , section          = section       ref `betterThen` sc
                    , chapterNumber    = chapterNumber ref `betterThen` ch
+                   , language         = language      ref `betterThen` ln
+                   , status           = status        ref `betterThen` st
                    , publisher        = fromAgent pu
                                            `betterThen` publisher ref
                                            `betterThen` fromAgent pu'
                                            `betterThen` fromAgent dg
                                            `betterThen` fromAgent om
+                                           `betterThen` fromAgent sp
+                   , originalDate           = issued         oref
+                   , originalTitle          = title          oref
+                   , originalPublisher      = publisher      oref
+                   , originalPublisherPlace = publisherPlace oref
                    }
-             , \r -> (  r
-                     , (refId     r, refType          r, (title r, titleShort r), isbn r, doi r)
-                     ,((author    r, editor           r, translator     r)
-                      ,(recipient r, interviewer      r, emptyAgents, director  r)
+             , \r -> ( (emptyReference,emptyReference)
+                     , (refId     r,(refType r,genre r), (title r, titleShort r), isbn r, doi r)
+                     ,((author    r, editor           r, translator r, director r)
+                      ,(recipient r, interviewer      r, emptyAgents,  director r)
                       ,(composer  r, collectionEditor r, emptyAgents, emptyAgents))
                      ,((issued    r, page  r, volume r, issue r)
-                      ,(number    r, section   r, chapterNumber r))
+                      ,(number    r, section   r, chapterNumber r, numberOfVolumes r))
                      , (issued    r, accessed r, emptyAgents, publisherPlace r, edition r)
-                     ,((accessed  r, url r), note r)
+                     ,((accessed  r, url r),  status r, language r, note r)
                      )) $
-      xp6Tuple (xpDefault emptyReference xpRelatedItem)
-               (xp5Tuple  xpCiteKey xpRefType xpTitle xpIsbn xpDoi)
-                xpAgents xpPart xpOrigin (xpPair xpUrl xpNote)
+      xp6Tuple (xpPair (xpDefault emptyReference $ xpRelatedItem "host")
+                       (xpDefault emptyReference $ xpRelatedItem "original"))
+               (xp5Tuple xpCiteKey xpRefType xpTitle xpIsbn xpDoi)
+                xpAgents xpPart xpOrigin
+               (xp4Tuple xpUrl xpLang xpStatus xpNote)
 
 xpCiteKey :: PU String
 xpCiteKey
@@ -116,9 +127,9 @@ xpOrigin
                (xpDefault [] $ xpIElem "place"   $ xpIElem "placeTerm" xpText0)
                (xpDefault [] $ xpIElem "edition" $                     xpText0)
 
-xpRefType :: PU RefType
+xpRefType :: PU (RefType, String)
 xpRefType
-    = xpDefault NoType $
+    = xpDefault (NoType,[]) $
       xpWrap (readRefType, const []) xpGenre
 
 xpGenre :: PU [String]
@@ -128,18 +139,23 @@ xpGenre
               (xpPair (xpDefault [] $ xpAttr "authority" xpText) xpText)
               $ xpLift . snd
 
-xpRelatedItem :: PU Reference
-xpRelatedItem
-    = xpIElem "relatedItem" . xpAddFixedAttr "type" "host" $
-      xpWrap ( \( (ty,ct)
-                ,((ca,ed,tr),(re,it,pu',dr),(co,ce,dg,om))
-                ,((di,pg,vl,is),(nu,sc,ch))
+xpRelatedItem :: String -> PU Reference
+xpRelatedItem t
+    = xpIElem "relatedItem" . xpAddFixedAttr "type" t $
+      xpWrap ( \(((t3l,t3s),(t4l,_))
+                ,((ty,gn),ct)
+                ,((ca,ed,tr,sp),(re,it,pu',dr),(co,ce,dg,om))
+                ,((di,pg,vl,is),(nu,sc,ch,vs))
                 , (di',ac,pu,pp,et)
+                , (ln, st)
                 ) ->
                emptyReference { refType             = ty
+                              , title               = fst ct
                               , containerAuthor     = ca
-                              , containerTitle      = fst ct
-                              , containerTitleShort = snd ct
+                              , containerTitle      = if t3l /= [] then t3l else fst ct
+                              , containerTitleShort = if t3s /= [] then t3s else snd ct
+                              , collectionTitle     = t4l
+                              , volumeTitle         = if t3l /= [] then fst ct else []
                               , editor              = ed
                               , edition             = et
                               , translator          = tr
@@ -157,20 +173,34 @@ xpRelatedItem
                               , number              = nu
                               , section             = sc
                               , chapterNumber       = ch
+                              , genre               = gn
+                              , numberOfVolumes     = vs
+                              , language            = ln
+                              , status              = st
                               , publisher           = fromAgent $ pu `betterThen` pu' `betterThen`
-                                                                  dg `betterThen` om
+                                                                  dg `betterThen` om  `betterThen` sp
 
                               }
-             , \r -> ( (refType         r, (containerTitle  r, containerTitleShort r))
-                     ,((containerAuthor r, editor           r, translator r)
-                      ,(recipient       r, interviewer      r, emptyAgents, director  r)
+             , \r -> (((volumeTitle r,[]),(collectionTitle r,[]))
+                     ,((refType r,genre r), (containerTitle  r, containerTitleShort r))
+                     ,((containerAuthor r, editor           r, translator r, director r)
+                      ,(recipient       r, interviewer      r, emptyAgents,  director r)
                       ,(composer        r, collectionEditor r, emptyAgents, emptyAgents))
                      ,((issued  r, page  r, volume r, issue r)
-                      ,(number  r, section   r, chapterNumber r))
+                      ,(number  r, section   r, chapterNumber r, numberOfVolumes r))
                      , (issued  r, accessed  r,emptyAgents, publisherPlace r, edition r)
+                     , (language r, status r)
                      )) $
-      xp4Tuple (xpPair  xpRefType xpTitle)
+      xp6Tuple  xpNestedTitles
+               (xpPair xpRefType xpTitle)
                 xpAgents xpPart xpOrigin
+               (xpPair xpLang xpStatus)
+
+xpNestedTitles :: PU ((String, String), (String, String))
+xpNestedTitles
+    = xpDefault (([],[]),([],[])) . getRelated $ xpPair xpTitle (getRelated xpTitle)
+    where
+      getRelated = xpIElem "relatedItem" . xpAddFixedAttr "type" "host"
 
 xpTitle :: PU (String,String)
 xpTitle
@@ -182,19 +212,22 @@ xpTitle
               xpAddFixedAttr "type" "abbreviated" $ xpElem "title" xpText0)
     where
       createTitle [] [] [] = ([],[])
-      createTitle s  [] [] = (s ,[])
+      createTitle s  [] [] = breakLong s
       createTitle s  [] ab = (s ,ab)
       createTitle s sub [] = (s ++ colon s ++ sub,  s)
       createTitle s sub ab = (s ++ colon s ++ sub, ab)
       colon s = if last s == '!' || last s == '?' then " " else ": "
+      breakLong s = let (a,b) = break (== ':') s
+                    in  if b /= [] then (s,a) else (s, [])
 
-xpAgents :: PU (([Agent],[Agent],[Agent])
+xpAgents :: PU (([Agent],[Agent],[Agent],[Agent])
                ,([Agent],[Agent],[Agent],[Agent])
                ,([Agent],[Agent],[Agent],[Agent]))
 xpAgents
-    = xpTriple (xpTriple (xpAgent "author"         "aut")
+    = xpTriple (xp4Tuple (xpAgent "author"         "aut")
                          (xpAgent "editor"         "edt")
-                         (xpAgent "translator"     "trl"))
+                         (xpAgent "translator"     "trl")
+                         (xpAgent "sponsor"        "spn"))
                (xp4Tuple (xpAgent "recipient"      "rcp")
                          (xpAgent "interviewer"    "ivr")
                          (xpAgent "publisher"      "pbl")
@@ -252,21 +285,22 @@ xpNameData
       readg = foldr (\(k,v) xs -> if k == "given"  then v:xs else xs) []
 
 xpPart :: PU (([RefDate],String,String,String)
-             ,(String,String,String))
+             ,(String,String,String,String))
 xpPart
     = xpDefault none . xpIElem "part" .
       xpWrap (readIt none,const []) $ xpList xpDetail
     where
-      none = (([],"","",""),("","",""))
+      none = (([],"","",""),("","","",""))
       readIt r [] = r
-      readIt acc@((d,p,v,i),(n,s,c)) (x:xs)
-          | Date      y <- x = readIt ((y,p,v,i),(n,s,c)) xs
-          | Page      y <- x = readIt ((d,y,v,i),(n,s,c)) xs
-          | Volume    y <- x = readIt ((d,p,y,i),(n,s,c)) xs
-          | Issue     y <- x = readIt ((d,p,v,y),(n,s,c)) xs
-          | Number    y <- x = readIt ((d,p,v,i),(y,s,c)) xs
-          | ChapterNr y <- x = readIt ((d,p,v,i),(n,s,y)) xs
-          | Section   y <- x = readIt ((d,p,v,i),(n,y,c)) xs
+      readIt acc@((d,p,v,i),(n,s,c,vs)) (x:xs)
+          | Date      y <- x = readIt ((y,p,v,i),(n,s,c,vs)) xs
+          | Page      y <- x = readIt ((d,y,v,i),(n,s,c,vs)) xs
+          | Volume    y <- x = readIt ((d,p,y,i),(n,s,c,vs)) xs
+          | Issue     y <- x = readIt ((d,p,v,y),(n,s,c,vs)) xs
+          | Number    y <- x = readIt ((d,p,v,i),(y,s,c,vs)) xs
+          | ChapterNr y <- x = readIt ((d,p,v,i),(n,s,y,vs)) xs
+          | Section   y <- x = readIt ((d,p,v,i),(n,y,c,vs)) xs
+          | NrVols    y <- x = readIt ((d,p,v,i),(n,s,c, y)) xs
           | otherwise        = acc
 
 data Detail
@@ -277,6 +311,7 @@ data Detail
     | Number    String
     | ChapterNr String
     | Section   String
+    | NrVols    String
       deriving ( Eq, Show )
 
 xpDetail :: PU Detail
@@ -286,6 +321,7 @@ xpDetail
       tag _ = 0
       ps = [ xpWrap (Date, const []) $ xpDate
            , xpWrap (Page,     show) $ xpPage
+           , xpWrap (NrVols,   show) $ xpVolumes
            , xpWrap (Volume,   show) $ xp "volume"
            , xpWrap (Issue,    show) $ xp "issue"
            , xpWrap (Number,   show) $ xp "number"
@@ -305,6 +341,11 @@ xpPage
                        (xpElem "end"   xpText))
                (\(s,e) -> xpLift (s ++ "-" ++ e))
 
+xpVolumes :: PU String
+xpVolumes
+    = xpElemWithAttrValue "extent" "unit" "volumes" $
+      xpElem "total" xpText
+
 xpUrl :: PU ([RefDate],String)
 xpUrl
     = xpDefault ([],[]) . xpIElem "location" $
@@ -325,6 +366,21 @@ xpIdentifier i
 xpNote :: PU (String)
 xpNote = xpDefault [] $ xpIElem "note" xpText
 
+xpLang :: PU String
+xpLang
+    = xpDefault [] $
+      xpChoice (xpIElem "recordInfo" $ xpIElem "languageOfCataloging" $
+                xpIElem "language" $ xpIElem "languageTerm"  xpText)
+               (xpIElem "recordInfo" $ xpIElem "languageOfCataloging" $
+                xpIElem "languageTerm"  xpText)
+                xpLift
+
+xpStatus :: PU String
+xpStatus
+    = xpDefault [] $
+      --xpElemWithAttrValue "note" "type" "publication status" xpText
+      xpIElem "note" $ xpAddFixedAttr "type" "publication status" xpText
+
 readDate :: String -> [RefDate]
 readDate s = (parseDate         $ takeWhile (/= '/') s) ++
              (parseDate . tail' $ dropWhile (/= '/') s)
@@ -345,12 +401,16 @@ emptyAgents  = []
 fromAgent :: [Agent] -> String
 fromAgent = foldr (<+>) [] . map show
 
-readRefType :: [String] -> RefType
-readRefType [] = NoType
-readRefType (t:_) =
+readRefType :: [String] -> (RefType, String)
+readRefType [] = (NoType,[])
+readRefType (t:ts) =
   case M.lookup t genreTypeMapping of
-    Just x  -> x
-    Nothing -> ArticleJournal       -- Reasonable default (?)
+    Just x  -> (x, if ts /= [] then head ts else [])
+    Nothing -> if ts /= []
+               then case M.lookup (head ts) genreTypeMapping of
+                      Just  x -> (x, t)
+                      Nothing -> (ArticleJournal, t)
+               else (ArticleJournal, [])
 
 -- The string constants come from http://www.loc.gov/standards/valuelist/marcgt.html, which are used in the
 -- "<genre></genre>" element (http://www.loc.gov/standards/mods/userguide/genre.html)
@@ -360,9 +420,10 @@ genreTypeMapping = M.fromList
   , ( "book chapter",               Chapter          )
   , ( "periodical",                 ArticleJournal   )
   , ( "newspaper",                  ArticleNewspaper )
+  , ( "magazine",                   ArticleNewspaper )
   , ( "magazine article",           ArticleNewspaper )
   , ( "encyclopedia",               EntryEncyclopedia)
-  , ( "conference publication",     PaperConference  )
+  , ( "conference publication",     Book             )
   , ( "academic journal",           ArticleJournal   )
   , ( "collection",                 Chapter          )
   , ( "legal case and case notes",  LegalCase        )
@@ -377,6 +438,7 @@ genreTypeMapping = M.fromList
   , ( "Ph.D. thesis",               Thesis           )
   , ( "Masters thesis",             Thesis           )
   , ( "report",                     Report           )
+  , ( "technical report",           Report           )
   , ( "review",                     Review           )
   , ( "thesis",                     Thesis           )
   , ( "unpublished",                NoType           )
