@@ -25,7 +25,7 @@ module Text.CSL.Output.Pandoc
     , endWithPunct
     ) where
 
-import Data.Char ( toUpper, toLower, isPunctuation )
+import Data.Char ( toUpper, toLower )
 import Data.Maybe ( fromMaybe )
 
 import Text.CSL.Style
@@ -37,26 +37,48 @@ import Text.Pandoc.Definition
 renderPandoc :: Style -> [FormattedOutput] -> [Inline]
 renderPandoc s
     = proc (convertQuoted s) . proc' (clean s $ isPunctuationInQuote s) .
-      concatMap (flipFlop . render s)
+      flipFlop . render s
 
 -- | Same as 'renderPandoc', but the output is wrapped in a pandoc
 -- paragraph block.
 renderPandoc' :: Style -> [FormattedOutput] -> Block
 renderPandoc' s
     = Para . proc (convertQuoted s) . proc' (clean s $ isPunctuationInQuote s) .
-      concatMap (flipFlop . render s)
+      flipFlop . render s
 
 -- | For the testsuite: we use 'Link' and 'Strikeout' to store
 -- "nocase" and "nodecor" rich text formatting classes.
 renderPandoc_ :: Style -> [FormattedOutput] -> [Inline]
 renderPandoc_ s
     = proc (convertQuoted s) . proc (clean' s $ isPunctuationInQuote s) .
-      concatMap (flipFlop . render s)
+      flipFlop . render s
 
-render :: Style -> FormattedOutput -> [Inline]
-render _ (FPan i) = i
-render _ (FDel s) = toStr s
-render sty fo
+render :: Style -> [FormattedOutput] -> [Inline]
+render _ [] = []
+render s (x:[])   = renderFo s x
+render s (x:y:os) = let a = renderFo s x
+                        b = renderFo s y
+                        isPunct = and . map (flip elem ".!?") in
+                    if isPunct (lastInline a) && isPunct (headInline b)
+                    then a ++ render s (tailFO [y] ++ os)
+                    else a ++ render s (y:os)
+
+tailFO :: [FormattedOutput] -> [FormattedOutput]
+tailFO [] = []
+tailFO (f:fs)
+    | FDel  s  <- f = FDel  (tail' s)       : fs
+    | FPan is  <- f = FPan  (tailInline is) : fs
+    | FN s  fm <- f = if prefix fm /= [] then FN s (tailFm fm)    : fs else FN    (tail' s) fm : fs
+    | FS s  fm <- f = if prefix fm /= [] then FS s (tailFm fm)    : fs else FS    (tail' s) fm : fs
+    | FO fm fo <- f = if prefix fm /= [] then FO   (tailFm fm) fo : fs else FO fm (tailFO fo)  : fs
+    | otherwise     = f : tailFO fs
+    where
+      tailFm fm = fm { prefix = tail $ prefix fm }
+
+renderFo :: Style -> FormattedOutput -> [Inline]
+renderFo _ (FPan i) = i
+renderFo _ (FDel s) = toStr s
+renderFo sty fo
     | FS str fm    <- fo = toPandoc fm $ toStr str
     | FN str fm    <- fo = toPandoc fm $ toStr $ rmZeros str
     | FO     fm xs <- fo = toPandoc fm $ rest xs
@@ -74,9 +96,11 @@ render sty fo
       toPandoc f i = addSuffix f $ toStr (prefix f) ++
                      (quote f . format f . proc cleanStrict $ i)
       format     f = font_variant f . font f . text_case f
-      rest      xs = procList xs $ concatMap (render sty)
-      quote    f i = if i /= [] && quotes f
-                     then [Quoted DoubleQuote . valign f $ i]
+      rest      xs = procList xs $ render sty
+      quote    f i = if i /= [] && quotes f /= NoQuote
+                     then if quotes f == NativeQuote
+                          then [escape "inquote"   . valign f $ i]
+                          else [Quoted DoubleQuote . valign f $ i]
                      else valign f i
       setCase f i
           | Str     s <- i = Str $ f s
@@ -85,8 +109,8 @@ render sty fo
           | Link s r <- i = Link (map (setCase f) s) r
           | otherwise     = setCase f i
 
-      toCap        s = if s /= []       then toUpper (head s) : map toLower (tail s) else []
-      toTitleCap   s = if isShortWord s then toUpper (head s) :              tail s  else s
+      toCap        s = if s /= []       then toUpper (head s) : tail s else []
+      toTitleCap   s = if isShortWord s then toUpper (head s) : tail s else s
       isShortWord  s = not $ s `elem` ["a","an","and","as","at","but","by","down","for","from"
                                       ,"in","into","nor","of","on","onto","or","over","so"
                                       ,"the","till","to","up","via","with","yet"]
@@ -174,19 +198,20 @@ clean s b (i:is)
 clean' :: Style -> Bool -> [Inline] -> [Inline]
 clean' _ _   []  = []
 clean' s b (i:is)
-    | Quoted t inls <- i
-    , b         = case headInline is of
-                    [x] -> if isPunctuation x
+    | Link inls (y,z) <- i, y == "inquote"
+    , y == z    = case headInline is of
+                    [x] -> if x `elem` ".," && b
                            then if lastInline inls `elem` [".",",",";",":","!","?"]
-                                then Quoted t (reverseQuoted t inls             ) : clean' s b (tailInline is)
-                                else Quoted t (reverseQuoted t inls ++ [Str [x]]) : clean' s b (tailInline is)
-                           else Quoted t (reverseQuoted t inls) : clean' s b  is
-                    _   -> Quoted t (reverseQuoted  t inls) : clean' s b is
-    | Quoted t inls <- i = Quoted t (reverseQuoted  t inls) : clean' s b is
+                                then quote DoubleQuote inls                : clean' s b (tailInline is)
+                                else quote DoubleQuote (inls ++ [Str [x]]) : clean' s b (tailInline is)
+                           else quote DoubleQuote inls : clean' s b is
+                    _   ->      quote DoubleQuote inls : clean' s b is
+    | Quoted t inls <- i = quote t inls : clean' s b is
     | otherwise = if lastInline [i] == headInline is && isPunct
                   then i : clean' s b (tailInline is)
                   else i : clean' s b is
     where
+      quote t x = Quoted t (reverseQuoted t x)
       isPunct = and . map (flip elem ".,;:!? ") $ headInline is
       reverseQuoted t = proc reverseQuoted'
           where
@@ -238,7 +263,8 @@ convertQuoted :: Style -> [Inline] -> [Inline]
 convertQuoted s = convertQuoted'
     where
       locale = let l = styleLocale s in case l of [x] -> x; _   -> Locale [] [] [] [] []
-      getQuote  x y = entityToChar . fst . fromMaybe (x,[]) . lookup (y,Long) . localeTermMap $ locale
+      getQuote  x y = entityToChar . termSingular . fromMaybe newTerm {termSingular = x} .
+                      findTerm y Long . localeTerms $ locale
       doubleQuotesO = getQuote "\"" "open-quote"
       doubleQuotesC = getQuote "\"" "close-quote"
       singleQuotesO = getQuote "'"  "open-inner-quote"
