@@ -45,6 +45,7 @@ render _ (FDel s) = s
 render b fo
     | (FS str fm   ) <- fo = prefix fm <++> format fm (trim   str    ) <++> suffix fm
     | (FN str fm   ) <- fo = prefix fm <++> format fm (trim   str    ) <++> suffix fm
+    | (FUrl t fm   ) <- fo = prefix fm <++> format fm (trim $ fst  t ) <++> suffix fm
     | (FO     fm xs) <- fo = prefix fm <++> format fm (trim $ rest xs) <++> suffix fm
     | otherwise            = []
     where
@@ -54,7 +55,7 @@ render b fo
       (<++>)    = if b then (++) else (<>)
       concatM f = foldr (<++>) [] . map f
 
-      quote  f s = if s /= [] && quotes f then "\"" ++ s ++ "\"" else s
+      quote  f s = if s /= [] && quotes f /= NoQuote then "\"" ++ s ++ "\"" else s
       capital  s = toUpper (head s) : (tail s)
       format f s = quote f . text_case f $ s
 
@@ -91,11 +92,12 @@ entityToChar s
     where
       parseEntity  = chr . readNum . takeWhile (/= ';') &&&
                      entityToChar . tail' . dropWhile (/= ';')
-      readNum ('x': n) = readNum $ "0x" ++ n
-      readNum       n  = case readsPrec 1 n of
-                           [(x,[])] -> x
-                           _        -> error $ "Invalid character entity:" ++ n
 
+readNum :: String -> Int
+readNum ('x': n) = readNum $ "0x" ++ n
+readNum       n  = case readsPrec 1 n of
+                     [(x,[])] -> x
+                     _        -> error $ "Invalid character entity:" ++ n
 
 head' :: [a] -> [a]
 head' = foldr (\x _ -> [x]) []
