@@ -151,9 +151,10 @@ instance XmlPickler Condition where
 
 instance XmlPickler Formatting where
     xpickle = xpWrap ( \(((p,s,ff),(fs,fv,fw)),(td,va,tc,d),(q,sp))
-                         -> Formatting p s ff fs fv fw td va tc d q sp False False
-                     , \(Formatting p s ff fs fv fw td va tc d q sp _ _)
-                         -> (((p,s,ff),(fs,fv,fw)),(td,va,tc,d),(q,sp))) $
+                         -> Formatting p s ff fs fv fw td va tc d
+                            (if q then NativeQuote else NoQuote) sp False False
+                     , \(Formatting p s ff fs fv fw td va tc d _ sp _ _)
+                         -> (((p,s,ff),(fs,fv,fw)),(td,va,tc,d),(False,sp))) $
               xpTriple (xpPair (xpTriple (xpAttrText' "prefix"      )
                                          (xpAttrText' "suffix"      )
                                          (xpAttrText' "font-family" ))
@@ -192,6 +193,9 @@ instance XmlPickler Sort where
 instance XmlPickler Bool where
     xpickle = xpWrap readable xpText
 
+instance XmlPickler Gender where
+    xpickle = xpWrap readable xpText
+
 instance XmlPickler Form where
     xpickle = xpWrap readable
                      (xpAttrWithDefault "long" "form" xpText)
@@ -290,15 +294,21 @@ xpLocale
                         xpTerms
                         (xpList xpLocaleDate))
 
-xpTerms :: PU [TermMap]
+xpTerms :: PU [CslTerm]
 xpTerms
     = xpWrap (concat,return) $ xpList $
       xpIElem "terms" $ xpList $ xpElem "term" $
-      xpPair (xpPair (xpAttrText "name") xpickle)
-             (xpChoice (xpWrap (\s -> (s,s), fst)  xpText0)
-                       (xpPair (xpIElem "single"   xpText0)
-                               (xpIElem "multiple" xpText0))
-                        xpLift)
+      xpWrap (\((n,f,g,gf,m),(s,p)) -> CT n f g gf s p m,
+             undefined) $
+             xpPair (xp5Tuple (xpAttrText "name")
+                              xpickle
+                              (xpAttrWithDefault Neuter "gender" xpickle)
+                              (xpAttrWithDefault Neuter "gender-form" xpickle)
+                              (xpAttrText' "match"))
+                    (xpChoice (xpWrap (\s -> (s,s), fst)  xpText0)
+                              (xpPair (xpIElem "single"   xpText0)
+                              (xpIElem "multiple" xpText0))
+                              xpLift)
 
 xpMacros :: PU [MacroMap]
 xpMacros
@@ -376,37 +386,41 @@ xpNameOpt
 
 xpBibOpts :: PU [Option]
 xpBibOpts
-    = xpWrap ( \((a,b,c,d,e),(f,g)) ->
+    = xpWrap ( \((a,b,c,d,e,f),(g,h)) ->
                  filter ((/=) [] . snd) $ [("hanging-indent",a)
                                           ,("second-field-align",b)
                                           ,("subsequent-author-substitute",c)
-                                          ,("line-spacing",d)
-                                          ,("entry-spacing",e)] ++ f ++ g
-                , const (([],[],[],[],[]),([],[]))) $
-      xpPair (xp5Tuple (xpAttrText' "hanging-indent")
+                                          ,("subsequent-author-substitute-rule",d)
+                                          ,("line-spacing",e)
+                                          ,("entry-spacing",f)] ++ g ++ h
+                , const (([],[],[],[],[],[]),([],[]))) $
+      xpPair (xp6Tuple (xpAttrText' "hanging-indent")
                        (xpAttrText' "second-field-align")
                        (xpAttrText' "subsequent-author-substitute")
+                       (xpAttrText' "subsequent-author-substitute-rule")
                        (xpAttrText' "line-spacing")
                        (xpAttrText' "entry-spacing")) $
              xpPair xpNameOpt xpNamesOpt
 
 xpCitOpts :: PU [Option]
 xpCitOpts
-    = xpWrap ( \((a,b,c),(d,e,f,g,h),(i,l)) ->
+    = xpWrap ( \((a,b,c),(d,e,f,g,h,i),(j,k)) ->
                  filter ((/=) [] . snd) $ [("disambiguate-add-names",a)
                                           ,("disambiguate-add-givenname",b)
                                           ,("disambiguate-add-year-suffix",c)
                                           ,("givenname-disambiguation-rule",d)
                                           ,("collapse",e)
-                                          ,("year-suffix-delimiter",f)
-                                          ,("after-collapse-delimiter",g)
-                                          ,("near-note-distance",h)] ++ i ++ l
-                , const (([],[],[]),([],[],[],[],[]),([],[]))) $
+                                          ,("cite-group-delimiter",f)
+                                          ,("year-suffix-delimiter",g)
+                                          ,("after-collapse-delimiter",h)
+                                          ,("near-note-distance",i)] ++ j ++ k
+                , const (([],[],[]),([],[],[],[],[],[]),([],[]))) $
       xpTriple (xpTriple (xpAttrText' "disambiguate-add-names")
                          (xpAttrText' "disambiguate-add-givenname")
                          (xpAttrText' "disambiguate-add-year-suffix"))
-               (xp5Tuple (xpAttrText' "givenname-disambiguation-rule")
+               (xp6Tuple (xpAttrText' "givenname-disambiguation-rule")
                          (xpAttrText' "collapse")
+                         (xpAttrText' "cite-group-delimiter")
                          (xpAttrText' "year-suffix-delimiter")
                          (xpAttrText' "after-collapse-delimiter")
                          (xpAttrText' "near-note-distance"))
@@ -567,16 +581,23 @@ langBase
       ,("en", "en-US")
       ,("es", "es-ES")
       ,("et", "et-EE")
+      ,("fa", "fa-IR")
+      ,("fi", "fi-FI")
       ,("fr", "fr-FR")
       ,("he", "he-IL")
+      ,("hr", "hr-HR")
       ,("hu", "hu-HU")
       ,("is", "is-IS")
       ,("it", "it-IT")
       ,("ja", "ja-JP")
+      ,("km", "km-KH")
       ,("ko", "ko-KR")
+      ,("lt", "lt-LT")
+      ,("lv", "lv-LV")
       ,("mn", "mn-MN")
       ,("nb", "nb-NO")
       ,("nl", "nl-NL")
+      ,("nn", "nn-NO")
       ,("pl", "pl-PL")
       ,("pt", "pt-PT")
       ,("ro", "ro-RO")
