@@ -15,13 +15,20 @@
 
 module Text.CSL.Style where
 
-import Data.List ( nubBy, isPrefixOf )
+import Control.Arrow
+import Data.List ( nubBy, isPrefixOf, isInfixOf )
 import Data.Generics ( Typeable, Data, everywhere
-                     , everywhere', everything, mkT, mkQ)
+                     , everywhere', everything, mkT, mkQ )
+import Data.Maybe ( listToMaybe )
 import qualified Data.Map as M
 import Text.JSON
 import Text.Pandoc.Definition ( Inline, Target )
 
+#ifdef UNICODE_COLLATION
+import qualified Data.Text     as T
+import qualified Data.Text.ICU as T
+#endif
+
 -- | The representation of a parsed CSL style.
 data Style
     = Style
@@ -42,7 +49,7 @@ data Locale
       { localeVersion :: String
       , localeLang    :: String
       , localeOptions :: [Option]
-      , localeTermMap :: [TermMap]
+      , localeTerms   :: [CslTerm]
       , localeDate    :: [Element]
       } deriving ( Show, Read, Eq, Typeable, Data )
 
@@ -57,20 +64,58 @@ mergeLocales s l ls = doMerge list
              filter ((\x -> x /= [] && x `isPrefixOf` s) . localeLang) ls ++
              filter ((==) [] . localeLang) ls
       doMerge x = return l { localeOptions = newOpt     x
-                           , localeTermMap = newTermMap x
+                           , localeTerms   = newTerms   x
                            , localeDate    = newDate    x
                            }
+      cht          = cslTerm &&& termForm &&& termGenderForm
+      checkedLoc   = if hasOrdinals ls then rmOrdinals (localeTerms l) else localeTerms l
+      newTerms   x = nubBy (\a b -> cht a == cht b) (concatMap localeTerms   x ++ checkedLoc)
       newOpt     x = nubBy (\a b -> fst a == fst b) (concatMap localeOptions x ++ localeOptions l)
-      newTermMap x = nubBy (\a b -> fst a == fst b) (concatMap localeTermMap x ++ localeTermMap l)
       newDate    x = nubBy (\(Date _ a _ _ _ _)
                              (Date _ b _ _ _ _) -> a == b) (concatMap localeDate x ++ localeDate l)
 
+data CslTerm
+    = CT
+      { cslTerm        :: String
+      , termForm       :: Form
+      , termGender     :: Gender
+      , termGenderForm :: Gender
+      , termSingular   :: String
+      , termPlural     :: String
+      , termMatch      :: String
+      } deriving ( Show, Read, Eq, Typeable, Data )
+
+newTerm :: CslTerm
+newTerm = CT [] Long Neuter Neuter [] [] []
+
+findTerm :: String -> Form -> [CslTerm] -> Maybe CslTerm
+findTerm s f
+    = listToMaybe . filter (cslTerm &&& termForm >>> (==) (s, f))
+
+findTerm' :: String -> Form -> Gender -> [CslTerm] -> Maybe CslTerm
+findTerm' s f g
+    = listToMaybe . filter (cslTerm &&& termForm &&& termGenderForm >>> (==) (s,(f,g)))
+
+hasOrdinals :: Data a => a -> Bool
+hasOrdinals = or . query hasOrd
+    where
+      hasOrd o
+          | CT {cslTerm = t} <- o
+          , "ordinal" `isInfixOf` t = [True]
+          | otherwise               = [False]
+
+rmOrdinals :: Data a => a -> a
+rmOrdinals = proc' doRemove
+    where
+      doRemove [] = []
+      doRemove (o:os)
+          | CT {cslTerm = t} <- o
+          , "ordinal" `isInfixOf` t =   doRemove os
+          | otherwise               = o:doRemove os
+
 type Abbrev
     = (String, [(String, M.Map String String)])
 
-type TermMap
-    = ((String,Form),(String,String))
-
 type MacroMap
     = (String,[Element])
 
@@ -180,10 +225,16 @@ instance Ord Sorting where
 compare' :: String -> String -> Ordering
 compare' x y
     = case (head x, head y) of
-        ('-','-') -> compare y x
+        ('-','-') -> comp y x
         ('-', _ ) -> LT
         (_  ,'-') -> GT
-        _         -> compare x y
+        _         -> comp x y
+      where
+#ifdef UNICODE_COLLATION
+        comp a b = T.collate (T.collator T.Current) (T.pack a) (T.pack b)
+#else
+        comp a b = compare a b
+#endif
 
 data Form
     = Long
@@ -195,6 +246,12 @@ data Form
     | NotSet
       deriving ( Eq, Show, Read, Typeable, Data )
 
+data Gender
+    = Feminine
+    | Masculine
+    | Neuter
+      deriving ( Eq, Show, Read, Typeable, Data )
+
 data NumericForm
     = Numeric
     | Ordinal
@@ -257,7 +314,7 @@ data Formatting
       , verticalAlign  :: String
       , textCase       :: String
       , display        :: String
-      , quotes         :: Bool
+      , quotes         :: Quote
       , stripPeriods   :: Bool
       , noCase         :: Bool
       , noDecor        :: Bool
@@ -265,9 +322,20 @@ data Formatting
 
 instance Show Formatting where show _ = "emptyFormatting"
 
+rmTitleCase :: Formatting -> Formatting
+rmTitleCase f
+    | Formatting _ _ _ _ _ _ _ _ "title" _ _ _ _ _ <- f = f {textCase = []}
+    | otherwise                                         = f
+
+data Quote
+    = NativeQuote
+    | ParsedQuote
+    | NoQuote
+    deriving ( Read, Eq, Ord, Typeable, Data )
+
 emptyFormatting :: Formatting
 emptyFormatting
-    = Formatting [] [] [] [] [] [] [] [] [] [] False False False False
+    = Formatting [] [] [] [] [] [] [] [] [] [] NoQuote False False False
 
 unsetAffixes :: Formatting -> Formatting
 unsetAffixes f = f {prefix = [], suffix = []}
@@ -285,7 +353,7 @@ mergeFM (Formatting aa ab ac ad ae af ag ah ai aj ak al am an)
                               (bh `betterThen` ah)
                               (bi `betterThen` ai)
                               (bj `betterThen` aj)
-                              (bk || ak)
+                              (if bk == NoQuote then ak else bk)
                               (bl || al)
                               (bm || am)
                               (bn || an)
@@ -322,6 +390,7 @@ data Output
     | OPan    [Inline]
     | ODel     String                                   -- ^ A delimiter string.
     | OStr     String             Formatting            -- ^ A simple 'String'
+    | OLabel   String             Formatting            -- ^ A label used for roles
     | ONum     Int                Formatting            -- ^ A number (used to count contributors)
     | OCitNum  Int                Formatting            -- ^ The citation number
     | ODate   [Output]                                  -- ^ A (possibly) ranged date
@@ -444,6 +513,8 @@ formatOutput o
     | ODel     s         <- o = FDel s
     | OStr     []      _ <- o = FNull
     | OStr     s       f <- o = FS s         f
+    | OLabel   []      _ <- o = FNull
+    | OLabel   s       f <- o = FS s         f
     | ODate    os        <- o = FO emptyFormatting (format os)
     | OYear    s _     f <- o = FS s         f
     | OYearSuf s _ _   f <- o = FS s         f
