@@ -24,7 +24,7 @@ module Text.CSL.Test
 import Control.Arrow
 import Control.Monad.State
 import Data.ByteString.Lazy.UTF8 ( fromString )
-import Data.Char (toLower)
+import Data.Char (toLower, chr)
 import Data.List
 import Data.Maybe (isJust)
 import Data.Time
@@ -242,13 +242,11 @@ runTest t = do
                  (_,       Just cs) -> cs
                  _                  -> [map (\r -> emptyCite { citeId = refId r }) $ testInput t]
       (BD cits bib) = citeproc opts style (testInput t) cites
-      fixCits = if isJust (testCitations t)
-                then flip (zipWith $ \c n -> ">>[" ++ show n ++ "] " ++ c) ([0..] :: [Int])
-                else id
-      output = case testMode t of
-                 "citation" -> unlines'  . fixCits . map (pandocToHTML . renderPandoc_ style) $ cits
+      output = superscript $
+               case testMode t of
+                 "citation" -> unlines'  . map (pandocToHTML . renderPandoc_ style) $ cits
                  _          -> pandocBib . map (pandocToHTML . renderPandoc_ style) $ bib
-  return (output == testResult t, output)
+  return (output == getResult t, output)
 
 test :: FilePath -> IO Bool
 test = doTest readJsonFile 0
@@ -270,7 +268,7 @@ doTest rf v f = do
                putStrLn' "++++++++++++++++++++++++++++++++++++++++++++++++++++++++"
                putStrLn' $ f ++ " failed!"
                putStrLn' "Expected:"
-               putStrLn' $ (testResult t)
+               putStrLn' $ getResult t
                putStrLn' "\nGot:"
                putStrLn' $ o
                when (v >= 3) $ putStrLn (show t)
@@ -301,3 +299,44 @@ runTS gs v f = do
   et <- getCurrentTime
   putStrLn $ (++) (formatTime defaultTimeLocale "%H:%M:%S" et) $ " <--------------END"
   putStrLn $ "Time: " ++ show (diffUTCTime et st)
+
+getResult :: Test -> String
+getResult t
+    = if isJust (testCitations t) && testMode t == "citation"
+      then unlines' . map  (\(a,b) -> drop (length (show b) + 5) a) .
+           flip zip ([0..] :: [Int]) . lines . testResult $ t
+      else testResult t
+
+superscript :: String -> String
+superscript [] = []
+superscript (x:xs)
+    = let a = lookup x (map (first (chr . readNum)) sups) in
+      case a of
+        Nothing -> x : superscript xs
+        Just x' -> "<sup>" ++ [chr $ readNum x'] ++ "</sup>" ++ superscript xs
+    where
+      sups = [("0x00AA","0x0061"),("0x00B2","0x0032"),("0x00B3","0x0033"),("0x00B9","0x0031")
+             ,("0x00BA","0x006F"),("0x02B0","0x0068"),("0x02B1","0x0266"),("0x02B2","0x006A")
+             ,("0x02B3","0x0072"),("0x02B4","0x0279"),("0x02B5","0x027B"),("0x02B6","0x0281")
+             ,("0x02B7","0x0077"),("0x02B8","0x0079"),("0x02E0","0x0263"),("0x02E1","0x006C")
+             ,("0x02E2","0x0073"),("0x02E3","0x0078"),("0x02E4","0x0295"),("0x1D2C","0x0041")
+             ,("0x1D2D","0x00C6"),("0x1D2E","0x0042"),("0x1D30","0x0044"),("0x1D31","0x0045")
+             ,("0x1D32","0x018E"),("0x1D33","0x0047"),("0x1D34","0x0048"),("0x1D35","0x0049")
+             ,("0x1D36","0x004A"),("0x1D37","0x004B"),("0x1D38","0x004C"),("0x1D39","0x004D")
+             ,("0x1D3A","0x004E"),("0x1D3C","0x004F"),("0x1D3D","0x0222"),("0x1D3E","0x0050")
+             ,("0x1D3F","0x0052"),("0x1D40","0x0054"),("0x1D41","0x0055"),("0x1D42","0x0057")
+             ,("0x1D43","0x0061"),("0x1D44","0x0250"),("0x1D45","0x0251"),("0x1D46","0x1D02")
+             ,("0x1D47","0x0062"),("0x1D48","0x0064"),("0x1D49","0x0065"),("0x1D4A","0x0259")
+             ,("0x1D4B","0x025B"),("0x1D4C","0x025C"),("0x1D4D","0x0067"),("0x1D4F","0x006B")
+             ,("0x1D50","0x006D"),("0x1D51","0x014B"),("0x1D52","0x006F"),("0x1D53","0x0254")
+             ,("0x1D54","0x1D16"),("0x1D55","0x1D17"),("0x1D56","0x0070"),("0x1D57","0x0074")
+             ,("0x1D58","0x0075"),("0x1D59","0x1D1D"),("0x1D5A","0x026F"),("0x1D5B","0x0076")
+             ,("0x1D5C","0x1D25"),("0x1D5D","0x03B2"),("0x1D5E","0x03B3"),("0x1D5F","0x03B4")
+             ,("0x1D60","0x03C6"),("0x1D61","0x03C7"),("0x2070","0x0030"),("0x2071","0x0069")
+             ,("0x2074","0x0034"),("0x2075","0x0035"),("0x2076","0x0036"),("0x2077","0x0037")
+             ,("0x2078","0x0038"),("0x2079","0x0039"),("0x207A","0x002B"),("0x207B","0x2212")
+             ,("0x207C","0x003D"),("0x207D","0x0028"),("0x207E","0x0029"),("0x207F","0x006E")
+             ,("0x3194","0x4E09"),("0x3195","0x56DB"),("0x3196","0x4E0A"),("0x3197","0x4E2D")
+             ,("0x3198","0x4E0B"),("0x3199","0x7532"),("0x319A","0x4E59"),("0x319B","0x4E19")
+             ,("0x319C","0x4E01"),("0x319D","0x5929"),("0x319E","0x5730"),("0x319F","0x4EBA")
+             ,("0x02C0","0x0294"),("0x02C1","0x0295"),("0x06E5","0x0648"),("0x06E6","0x064A")]
